@@ -57,6 +57,7 @@
 #include <QStyleHints>
 
 #include <QDebug>
+#include <QToolTip>
 
 using namespace Peony;
 using namespace Peony::DirectoryView;
@@ -70,6 +71,23 @@ IconView::IconView(QWidget *parent) : QListView(parent)
     //FIXME: do not create proxy in view itself.
     IconViewDelegate *delegate = new IconViewDelegate(this);
     setItemDelegate(delegate);
+    connect(delegate, &IconViewDelegate::isEditing, this, [=](const bool &editing)
+    {
+        m_delegate_editing = editing;
+    });
+
+    //fix long file name not fully painted issue when drag sliderbar
+    QScrollBar *verticalBar = verticalScrollBar();
+    connect(verticalBar, &QScrollBar::sliderPressed, [=](){
+        m_slider_bar_draging = true;
+    });
+    connect(verticalBar, &QScrollBar::sliderReleased, [=](){
+        m_slider_bar_draging = false;
+    });
+    connect(verticalBar, &QScrollBar::valueChanged, [=](){
+        if (m_slider_bar_draging)
+            viewport()->update(viewport()->rect());
+    });
 
     setSelectionMode(QListView::ExtendedSelection);
     setEditTriggers(QListView::NoEditTriggers);
@@ -83,10 +101,12 @@ IconView::IconView(QWidget *parent) : QListView(parent)
     setGridSize(QSize(115, 135));
     setIconSize(QSize(64, 64));
 
-
     m_renameTimer = new QTimer(this);
     m_renameTimer->setInterval(3000);
+    m_renameTimer->setSingleShot(true);
     m_editValid = false;
+
+    setMouseTracking(true);//追踪鼠标
 }
 
 IconView::~IconView()
@@ -181,7 +201,7 @@ void IconView::closeView()
 void IconView::dragEnterEvent(QDragEnterEvent *e)
 {
     m_editValid = false;
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
@@ -196,13 +216,24 @@ void IconView::dragEnterEvent(QDragEnterEvent *e)
 
 void IconView::dragMoveEvent(QDragMoveEvent *e)
 {
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
 
+    //fix can not drag in the second time issue
+    if (this->isDraggingState())
+    {
+        if (m_allow_set_index_widget) {
+            m_allow_set_index_widget = false;
+            for (auto index : selectedIndexes()) {
+                setIndexWidget(index, nullptr);
+            }
+        }
+    }
+
     auto action = m_ctrl_key_pressed ? Qt::CopyAction : Qt::MoveAction;
-    qDebug()<<"dragMoveEvent()" <<action <<m_ctrl_key_pressed;
+    //qDebug()<<"dragMoveEvent()" <<action <<m_ctrl_key_pressed;
     auto index = indexAt(e->pos());
     if (index.isValid() && index != m_last_index) {
         QHoverEvent he(QHoverEvent::HoverMove, e->posF(), e->posF());
@@ -222,7 +253,7 @@ void IconView::dropEvent(QDropEvent *e)
 {
     m_last_index = QModelIndex();
     //m_edit_trigger_timer.stop();
-    if (e->keyboardModifiers() && Qt::ControlModifier)
+    if (e->keyboardModifiers() & Qt::ControlModifier)
         m_ctrl_key_pressed = true;
     else
         m_ctrl_key_pressed = false;
@@ -236,7 +267,15 @@ void IconView::dropEvent(QDropEvent *e)
     if (e->source() == this)
     {
         if (indexAt(e->pos()).isValid())
-            m_model->dropMimeData(e->mimeData(), action, 0, 0, index);
+        {
+            auto uri = m_sort_filter_proxy_model->itemFromIndex(proxy_index)->uri();
+            if(!e->mimeData()->urls().contains(uri))
+                m_model->dropMimeData(e->mimeData(), action, 0, 0, index);
+        } else {
+            if (m_ctrl_key_pressed) {
+                m_model->dropMimeData(e->mimeData(), Qt::CopyAction, 0, 0, QModelIndex());
+            }
+        }
         return;
     }
 
@@ -245,6 +284,13 @@ void IconView::dropEvent(QDropEvent *e)
 
 void IconView::mouseMoveEvent(QMouseEvent *e)
 {
+    QModelIndex itemIndex = indexAt(e->pos());
+    if (!itemIndex.isValid()) {
+        if (QToolTip::isVisible()) {
+            QToolTip::hideText();
+        }
+    }
+
     if (m_ignore_mouse_move_event) {
         return;
     }
@@ -253,6 +299,8 @@ void IconView::mouseMoveEvent(QMouseEvent *e)
 
 void IconView::mousePressEvent(QMouseEvent *e)
 {
+    m_allow_set_index_widget = true;
+
     qDebug()<<"moursePressEvent";
     m_editValid = true;
     QListView::mousePressEvent(e);
@@ -279,7 +327,6 @@ void IconView::mousePressEvent(QMouseEvent *e)
             m_editValid = false;
         }
     }
-
 }
 
 void IconView::mouseReleaseEvent(QMouseEvent *e)
@@ -347,8 +394,40 @@ void IconView::updateGeometries()
     verticalScrollBar()->setMaximum(verticalScrollBar()->maximum() + BOTTOM_STATUS_MARGIN);
 }
 
+void IconView::focusInEvent(QFocusEvent *e)
+{
+    QListView::focusInEvent(e);
+    if (e->reason() == Qt::TabFocus) {
+        if (selectedIndexes().isEmpty()) {
+            selectionModel()->select(model()->index(0, 0), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
+        } else {
+            scrollTo(selectedIndexes().first(), QListView::PositionAtCenter);
+            auto selections = selectedIndexes();
+            clearSelection();
+            QTimer::singleShot(100, this, [=](){
+                for (auto index : selections) {
+                    selectionModel()->select(index, QItemSelectionModel::Select);
+                }
+            });
+        }
+    }
+}
+
 void IconView::slotRename()
 {
+    //special path like trash path not allow rename
+    if (getDirectoryUri().startsWith("trash://")
+        || getDirectoryUri().startsWith("recent://")
+        || getDirectoryUri().startsWith("favorite://")
+        || getDirectoryUri().startsWith("search://"))
+        return;
+
+    //standardPaths not allow rename
+    auto currentSelections = getSelections();
+    bool hasStandardPath = FileUtils::containsStandardPath(currentSelections);
+    if (hasStandardPath)
+        return;
+
     //delay edit action to avoid doubleClick or dragEvent
     qDebug()<<"slotRename"<<m_editValid;
     QTimer::singleShot(300, m_renameTimer, [&]() {
@@ -389,6 +468,9 @@ void IconView::bindModel(FileItemModel *sourceModel, FileItemProxyFilterSortMode
             this->setIndexWidget(index, nullptr);
         }
 
+        if (state() == QListView::DragSelectingState)
+            m_allow_set_index_widget = false;
+
         //Q_EMIT m_proxy->viewSelectionChanged();
         if (currentSelections.count() == 1) {
             qDebug()<<"m_last_index  "<<(m_last_index == currentSelections.first())<<currentSelections.first();
@@ -425,9 +507,13 @@ void IconView::setProxy(DirectoryViewProxyIface *proxy)
     connect(m_model, &FileItemModel::findChildrenFinished,
             this, &IconView::reportViewDirectoryChanged);
 
-    connect(this, &IconView::doubleClicked, [=](const QModelIndex &index) {
+    connect(this, &IconView::activated, [=](const QModelIndex &index) {
         qDebug()<<"double click"<<index.data(FileItemModel::UriRole);
-        Q_EMIT m_proxy->viewDoubleClicked(index.data(FileItemModel::UriRole).toString());
+        //when selections is more than 1, let mainwindow to process
+        if (getSelections().count() != 1)
+            return;
+        auto uri = getSelections().first();
+        Q_EMIT m_proxy->viewDoubleClicked(uri);
     });
 
 
@@ -504,11 +590,14 @@ const QStringList IconView::getAllFileUris()
 
 void IconView::editUri(const QString &uri)
 {
+    setState(QListView::NoState);
     auto origin = FileUtils::getOriginalUri(uri);
     setIndexWidget(m_sort_filter_proxy_model->indexFromUri(origin), nullptr);
     qDebug() <<"editUri:" <<uri <<origin;
     QListView::scrollTo(m_sort_filter_proxy_model->indexFromUri(origin));
     edit(m_sort_filter_proxy_model->indexFromUri(origin));
+//    if (! m_delegate_editing)
+//        edit(m_sort_filter_proxy_model->indexFromUri(origin));
 }
 
 void IconView::editUris(const QStringList uris)
@@ -522,6 +611,7 @@ void IconView::clearIndexWidget()
     for (int i = 0; i < m_sort_filter_proxy_model->rowCount(); i++) {
         auto index = m_sort_filter_proxy_model->index(i, 0);
         setIndexWidget(index, nullptr);
+        closePersistentEditor(index);
     }
 }
 
@@ -557,6 +647,8 @@ void IconView2::bindModel(FileItemModel *model, FileItemProxyFilterSortModel *pr
     m_proxy_model = proxyModel;
 
     m_view->bindModel(model, proxyModel);
+    connect(m_model, &FileItemModel::selectRequest, this, &DirectoryViewWidget::updateWindowSelectionRequest);
+
     connect(model, &FileItemModel::findChildrenFinished, this, &DirectoryViewWidget::viewDirectoryChanged);
     //connect(m_model, &FileItemModel::dataChanged, m_view, &IconView::clearIndexWidget);
     connect(m_model, &FileItemModel::updated, m_view, &IconView::resort);
@@ -564,8 +656,12 @@ void IconView2::bindModel(FileItemModel *model, FileItemProxyFilterSortModel *pr
     connect(m_view->selectionModel(), &QItemSelectionModel::selectionChanged,
             this, &DirectoryViewWidget::viewSelectionChanged);
 
-    connect(m_view, &IconView::doubleClicked, this, [=](const QModelIndex &index) {
-        Q_EMIT this->viewDoubleClicked(index.data(Qt::UserRole).toString());
+    connect(m_view, &IconView::activated, this, [=](const QModelIndex &index) {
+        //when selections is more than 1, let mainwindow to process
+        if (getSelections().count() != 1)
+            return;
+        auto uri = getSelections().first();
+        Q_EMIT this->viewDoubleClicked(uri);
     });
 
     connect(m_view, &IconView::customContextMenuRequested, this, [=](const QPoint &pos) {
@@ -620,7 +716,8 @@ void IconView2::setCurrentZoomLevel(int zoomLevel)
 
 void IconView2::clearIndexWidget()
 {
-    for (auto index : m_view->selectedIndexes()) {
+    for (auto index : m_proxy_model->getAllFileIndexes()) {
+        m_view->closePersistentEditor(index);
         m_view->setIndexWidget(index, nullptr);
     }
 }
