@@ -152,6 +152,8 @@ protected:
 
     void updateGeometries() override;
 
+    void focusInEvent(QFocusEvent *e) override;
+
     bool getIgnore_mouse_move_event() const;
     void setIgnore_mouse_move_event(bool ignore_mouse_move_event);
 
@@ -183,6 +185,12 @@ private:
     const int BOTTOM_STATUS_MARGIN = 36;
 
     bool m_ignore_mouse_move_event = false;
+
+    bool m_delegate_editing = false;
+
+    bool m_allow_set_index_widget = true;
+
+    bool m_slider_bar_draging = false;
 };
 
 //IconView2
