@@ -52,6 +52,7 @@ class FileInfo;
 class PEONYCORESHARED_EXPORT FileInfoJob : public QObject
 {
     friend class FileInfo;
+    friend class FileEnumerator;
 
     Q_OBJECT
 public:
@@ -122,10 +123,22 @@ protected:
             FileInfoJob *thisJob);
 
 private:
-    void refreshInfoContents(GFileInfo *new_info);
+    void refreshFileSystemInfo (GFileInfo* new_info);
+    void refreshInfoContents (GFileInfo *new_info);
     std::shared_ptr<FileInfo> m_info;
 
+    quint64 m_file_size_uint = 0;
+    quint64 m_file_modify_uint = 0;
+
+    QString m_file_name = nullptr;
+    QString m_file_size = nullptr;
+    QString m_file_modify = nullptr;
+    QString m_file_location = nullptr;
+
     bool m_auto_delete = false;
+
+    GCancellable *m_cancellable = nullptr;
+    GCancellable *m_fs_cancellable = nullptr;
 };
 
 }
