@@ -41,6 +41,54 @@ namespace Peony {
 class FileInfoJob;
 class FileMetaInfo;
 
+static char *office_mime_types[] = {
+    "application/wps-office.doc",
+    "application/msword",
+    "application/vnd.ms-word",
+    "application/x-msword",
+    "application/vnd.ms-word.document.macroenabled.12",
+    "application/wps-office.dot",
+    "application/msword-template",
+    "application/vnd.ms-word.template.macroenabled.12",
+    "application/wps-office.dotx",
+    "application/vnd.openxmlformats-officedocument.wordprocessingml.template",
+    "application/wps-office.docx",
+    "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
+    "application/wps-office.wps",
+    "application/vnd.ms-works",
+
+    "application/wps-office.wpt",
+    "application/wps-office.ppt",
+    "application/vnd.ms-powerpoint",
+    "application/powerpoint",
+    "application/mspowerpoint",
+    "application/x-mspowerpoint",
+    "application/vnd.ms-powerpoint.presentation.macroenabled.12",
+    "application/wps-office.pot",
+    "application/vnd.ms-powerpoint.template.macroenabled.12",
+    "application/wps-office.potx",
+    "application/vnd.openxmlformats-officedocument.presentationml.template",
+    "application/wps-office.pptx",
+    "application/vnd.openxmlformats-officedocument.presentationml.presentation",
+    "application/vnd.openxmlformats-officedocument.presentationml.slideshow",
+    "application/wps-office.dps",
+    "application/wps-office.dpt",
+
+    "application/wps-office.xls",
+    "application/vnd.ms-excel",
+    "application/msexcel",
+    "application/x-msexcel",
+    "application/vnd.ms-excel.sheet.macroenabled.12",
+    "application/vnd.ms-excel.template.macroenabled.12",
+    "application/wps-office.xlt",
+    "application/wps-office.xltx",
+    "application/vnd.openxmlformats-officedocument.spreadsheetml.template",
+    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
+    "application/wps-office.et",
+    "application/wps-office.ett",
+    "end"
+};
+
 /*!
  * \brief The FileInfo class
  * <br>
@@ -71,9 +119,30 @@ public:
     explicit FileInfo(QObject *parent = nullptr);
     explicit FileInfo(const QString &uri, QObject *parent = nullptr);
     ~FileInfo();
-    static std::shared_ptr<FileInfo> fromUri(QString uri, bool addToHash = true);
-    static std::shared_ptr<FileInfo> fromPath(QString path, bool addToHash = true);
-    static std::shared_ptr<FileInfo> fromGFile(GFile *file, bool addToHash = true);
+    /*!
+     * \brief fromUri
+     * \param uri
+     * \param addToHash
+     * \return
+     * \deprecated
+     */
+    static std::shared_ptr<FileInfo> fromUri(QString uri);
+    /*!
+     * \brief fromPath
+     * \param path
+     * \param addToHash
+     * \return
+     * \deprecated
+     */
+    static std::shared_ptr<FileInfo> fromPath(QString path);
+    /*!
+     * \brief fromGFile
+     * \param file
+     * \param addToHash
+     * \return
+     * \deprecated
+     */
+    static std::shared_ptr<FileInfo> fromGFile(GFile *file);
 
     QString uri() {
         return m_uri;
@@ -90,10 +159,15 @@ public:
     bool isVirtual() {
         return m_is_virtual;
     }
-
-    QString displayName() {
-        return m_display_name;
+    bool isValid()
+    {
+        return m_is_valid;
     }
+
+    QString desktopName(){
+        return m_desktop_name;
+    }
+
     QString iconName() {
         return m_icon_name;
     }
@@ -127,6 +201,11 @@ public:
     QString type() {
         return m_content_type;
     }
+
+    QString fileSystemType() {
+        return m_fs_type;
+    }
+
     quint64 size() {
         return m_size;
     }
@@ -179,8 +258,24 @@ public:
     bool isDesktopFile() {
         return m_can_excute && m_uri.endsWith(".desktop");
     }
+
+    bool isPdfFile(){
+        return m_mime_type_string.contains("pdf");
+    }
+
+    bool isImageFile(){
+        return m_mime_type_string.startsWith("image/");
+    }
+
+    bool isImagePdfFile(){
+        return m_mime_type_string.contains("djvu");
+    }
+
+    bool isVideoFile();
+    bool isOfficeFile();
+
     bool isEmptyInfo() {
-        return m_display_name == nullptr;
+        return m_display_name == nullptr || m_display_name == "";
     }
 
     AccessFlags accesses() {
@@ -214,6 +309,13 @@ public:
         return m_file;
     }
 
+    const QString targetUri();
+    const QString displayName();
+    const QString symlinkTarget();
+    const QString unixDeviceFile();
+
+    const QString customIcon();
+
     //const QIcon thumbnail() {return m_thumbnail;}
     //void setThumbnail(const QIcon &thumbnail) {m_thumbnail = thumbnail;}
 
@@ -232,6 +334,7 @@ private:
     bool m_is_loaded = false;
 
     QString m_display_name = nullptr;
+    QString m_desktop_name = nullptr;
     QString m_icon_name = nullptr;
     QString m_symbolic_icon_name = nullptr;
     QString m_file_id = nullptr;
@@ -266,12 +369,17 @@ private:
 
     bool m_can_start = false;
     bool m_can_stop = false;
+    const char *m_unix_device_file = nullptr;
 
     //FIXME: should i use smart pointer wrap these data?
     GFile *m_file = nullptr;
     GFile *m_parent = nullptr;
 
-    GFile *m_target_file = nullptr;
+    QString m_target_uri;
+    QString m_symlink_target;
+
+    // filesystem
+    QString m_fs_type;
 
     /*!
      * \brief m_cancellable
