@@ -52,6 +52,7 @@ public:
     }
 
     void createThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher = nullptr, bool force = false);
+    void clearThumbnail();
     void releaseThumbnail(const QString &uri);
     void updateDesktopFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher = nullptr);
     const QIcon tryGetThumbnail(const QString &uri);
@@ -69,6 +70,16 @@ private:
     ~ThumbnailManager();
     void createThumbnailInternal(const QString &uri, std::shared_ptr<FileWatcher> watcher = nullptr, bool force = false);
 
+    void createVideFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+    void createPdfFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+    void createImageFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+    void createOfficeFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+    void createDesktopFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+
+    //djvu file process
+    void findAtril();
+    void createImagePdfFileThumbnail(const QString &uri, std::shared_ptr<FileWatcher> watcher);
+
     QHash<QString, QIcon> m_hash;
     //QMutex m_mutex;
 
